<?php
/**
 * This file is part of the official Amazon Checkout extension for Magento 1.x
 *
 * (c) 2020 creativestyle GmbH. All Rights reserved
 *
 * Distribution of the derivatives reusing, transforming or being built upon
 * this software, is not allowed without explicit written permission granted
 * by creativestyle GmbH
 *
 * @category   Creativestyle
 * @package    Creativestyle_AmazonCheckout
 * @copyright  2020 creativestyle GmbH
 * @author     Marek Stefan <ticket@creativestyle.de>
 */
class Creativestyle_AmazonCheckout_Block_Adminhtml_Info extends Mage_Adminhtml_Block_System_Config_Form_Fieldset
{
    const REGISTRATION_BLOCK_ID = 'amazonpay-simple-path-registration';
    const REQUIREMENTS_BLOCK_ID = 'amazonpay-requirements-check';
    const DOC_BASE_URL = 'https://amazon-pay.rtfd.io/en/latest/%s';

    /**
     * Return instance of Amazon Pay config object
     *
     * @return Creativestyle_AmazonCheckout_Model_Config
     */
    protected function _getConfig()
    {
        /** @var Creativestyle_AmazonCheckout_Model_Config $config */
        $config = Mage::getSingleton('amazonpay/config');
        return $config;
    }

    /**
     * Return Amazon Pay debug model instance
     *
     * @return Creativestyle_AmazonCheckout_Model_Debug
     */
    protected function _getDebugModel()
    {
        /** @var Creativestyle_AmazonCheckout_Model_Debug $debugModel */
        $debugModel = Mage::getSingleton('amazonpay/debug');
        return $debugModel;
    }

    public function render(Varien_Data_Form_Element_Abstract $element)
    {
        $this->setTemplate('creativestyle/amazonpay/info.phtml');
        return $this->toHtml();
    }

    /**
     * Return current version of the extension
     *
     * @return string
     */
    public function getExtensionVersion()
    {
        return $this->_getConfig()->getVersion();
    }

    /**
     * Return HTML ID of the registration block
     *
     * @return string
     */
    public function getRegistrationBlockId()
    {
        return self::REGISTRATION_BLOCK_ID;
    }

    /**
     * Return HTML ID of the requirements check block
     *
     * @return string
     */
    public function getRequirementsBlockId()
    {
        return self::REQUIREMENTS_BLOCK_ID;
    }

    /**
     * Check whether merchant account is registered
     *
     * @return boolean
     */
    public function isRegistered()
    {
        if ($this->_getConfig()->getMerchantId()) {
            return true;
        }

        return false;
    }

    /**
     * Check whether legacy Amazon Payments extension is installed
     *
     * @return boolean
     */
    public function isLegacyExtensionInstalled()
    {
        return $this->_getConfig()->isLegacyExtensionInstalled();
    }

    public function useLegacyExtension()
    {
        return $this->_getConfig()->useLegacyExtension();
    }

    /**
     * Return array of available account regions
     *
     * @return array
     */
    public function getAccountRegions()
    {
        return Mage::getSingleton('amazonpay/system_config_source_region')->toOptionArray();
    }

    public function getDefaultAccountRegion()
    {
        return Mage::getStoreConfig('currency/options/base');
    }

    /**
     * Return URL to the requirements section of the documentation
     *
     * @return string
     */
    public function getRequirementsDocUrl()
    {
        return sprintf(self::DOC_BASE_URL, 'troubleshooting.html#installing-the-extension-on-magento-lower-than-1-9-4-0');
    }

    /**
     * Return concatenated list of Magento patches required to run Amazon Pay
     *
     * @return string
     */
    public function getRequiredMagentoPatches()
    {
        return join(', ', $this->_getDebugModel()->getRequiredPatches());
    }

    /**
     * Return array of Magento patches missing to run Amazon Pay
     *
     * @return array
     */
    public function getMissingMagentoPatches()
    {
        return $this->_getDebugModel()->getMissingPatches();
    }

    /**
     * Return result of phpseclib validation
     *
     * @return bool
     */
    public function isPhpseclibValid()
    {
        return $this->_getDebugModel()->validatePhpseclib();
    }
}
