<?php
/**
 * This file is part of the official Amazon Checkout extension for Magento 1.x
 *
 * (c) 2020 creativestyle GmbH. All Rights reserved
 *
 * Distribution of the derivatives reusing, transforming or being built upon
 * this software, is not allowed without explicit written permission granted
 * by creativestyle GmbH
 *
 * @category   Creativestyle
 * @package    Creativestyle_AmazonCheckout
 * @copyright  2020 creativestyle GmbH
 * @author     Marek Stefan <ticket@creativestyle.de>
 */

class Creativestyle_AmazonCheckout_Helper_Data extends Mage_Core_Helper_Abstract
{
    /**
     * Return instance of Amazon Pay config object
     *
     * @return Creativestyle_AmazonCheckout_Model_Config
     */
    protected function _getConfig()
    {
        /** @var Creativestyle_AmazonCheckout_Model_Config $config */
        $config = Mage::getSingleton('amazonpay/config');
        return $config;
    }

    /**
     * Return Amazon Pay debug model instance
     *
     * @return Creativestyle_AmazonCheckout_Model_Debug
     */
    protected function _getDebugModel()
    {
        /** @var Creativestyle_AmazonCheckout_Model_Debug $debugModel */
        $debugModel = Mage::getSingleton('amazonpay/debug');
        return $debugModel;
    }

    /**
     * Return flag whether Amazon Pay runs in sandbox mode
     *
     * @param mixed|null $store
     * @return bool
     * @api
     */
    public function isSandboxActive($store = null)
    {
        /** @var Creativestyle_AmazonCheckout_Model_System_Config_Source_Environment $envSource */
        $envSource = Mage::getSingleton('amazonpay/system_config_source_environment');
        return $envSource->isSandbox($this->_getConfig()->getEnvironment($store));
    }

    /**
     * Split customer name into first- and last name
     * and return result as an object
     *
     * @param string $customerName
     * @param string|null $emptyValuePlaceholder
     * @return Varien_Object
     */
    public function splitCustomerName($customerName, $emptyValuePlaceholder = 'n/a')
    {
        $splitName = explode(' ', trim($customerName));
        $result = array();
        if (count($splitName) > 1) {
            $result['firstname'] = array_shift($splitName);
            $result['lastname'] = implode(' ', $splitName);
        } else {
            $result['firstname'] = null !== $emptyValuePlaceholder
                ? Mage::helper('amazonpay')->__($emptyValuePlaceholder) : null;
            $result['lastname'] = reset($splitName);
        }

        return new Varien_Object($result);
    }

    /**
     * Generate and return UUID v4
     *
     * @return string
     * @api
     */
    public function generateUuid()
    {
        return sprintf('%04x%04x%04x%04x%04x%04x%04x%04x',
            mt_rand(0, 0xffff),
            mt_rand(0, 0xffff),
            mt_rand(0, 0xffff),
            mt_rand(0, 0x0fff) | 0x4000,
            mt_rand(0, 0x3fff) | 0x8000,
            mt_rand(0, 0xffff),
            mt_rand(0, 0xffff),
            mt_rand(0, 0xffff)
        );
    }

    /**
     * @api
     */
    public function getAvailablePaymentMethods()
    {
        return array('amazon_checkout');
    }

    /**
     * @param string $key
     * @return bool
     * @api
     */
    public function isPrivateKeyValid($key = null)
    {
        return ((strpos($key, 'BEGIN RSA PRIVATE KEY') !== false) || (strpos($key, 'BEGIN PRIVATE KEY') !== false)) &&
            $this->_getDebugModel()->validatePhpseclib();
    }

    /**
     * @param array $customFields
     * @return array
     * @api
     */
    public function filterCheckoutCustomFields($customFields)
    {
        $filteredCustomFields = array();
        if (!empty($customFields)) {
            $allowedFields = $this->_getConfig()->getCheckoutCustomFields();

            foreach ($customFields as $customFieldName => $customFieldValue) {
                if (isset($allowedFields[$customFieldName])) {
                    if (is_array($customFieldValue)) {
                        if (in_array($customFieldName, array('customer', 'billing_address', 'shipping_address'))) {
                            $filteredCustomFields[$customFieldName] = array();
                            foreach ($customFieldValue as $key => $value) {
                                if (isset($allowedFields[$customFieldName][$key])) {
                                    $filteredCustomFields[$customFieldName][$key] = $value;
                                }
                            }
                        }
                    } else {
                        $filteredCustomFields[$customFieldName] = $customFieldValue;
                    }
                }
            }
        }

        return $filteredCustomFields;
    }
}
